#include <sys/types.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include "ora_date.h"

/* ---------------------------------------------------------------------------
   Function                 : julian
   Summary                  : Take a date in given format
			      and give the equivalent Julian date.
   Input Parameters         : The date string
   			      The format
   Output Parameters        : None
   Side effects             : None
   Return Value             : julian date or zero
   --------------------------------------------------------------------------*/

long julian(char *dt_str, char *fmt)
{
  time_t  t;
  char   *ch;
  char   *where;

  if (!fmt || !fmt[0])
    fmt = "DD-MON-YY";
  ch = (char *)NULL;
  if (date_val(dt_str, fmt, &where, &t) != (struct tm *)NULL)
    ch = to_char(t, "J");

  return (!ch) ? 0 : atol(ch);
}

/* ---------------------------------------------------------------------------
   Function                 : duration
   Summary                  : Take two standard dates in funny format
			      YYYYMMDDHH24MI[SS] and the allowance minutes and
			      determine duration.
   Input Parameters         : The start date string
			      The end   date string
			      The allowance minutes
   Output Parameters        : None
   Side effects             : None
   Return Value             : number of days (-1 is error)
   --------------------------------------------------------------------------*/

int duration(char *st_dt, char *nd_dt, int allowance)
{
  time_t  st, nd;
  char   *where;
  int     days;
  char   *st_fmt = (char *)NULL;
  char   *nd_fmt = (char *)NULL;

  if      (strlen(st_dt) == 12)
    st_fmt = "YYYYMMDDHH24MI";
  else if (strlen(st_dt) == 14)
    st_fmt = "YYYYMMDDHH24MISS";

  if      (strlen(nd_dt) == 12)
    nd_fmt = "YYYYMMDDHH24MI";
  else if (strlen(nd_dt) == 14)
    nd_fmt = "YYYYMMDDHH24MISS";

  if (!st_fmt || !nd_fmt)
    return -1;

  days = 0;

  if (date_val(st_dt, st_fmt, &where, &st) != (struct tm *)NULL &&
      date_val(nd_dt, nd_fmt, &where, &nd) != (struct tm *)NULL)
  {
    nd -= allowance * 60L;
    if (nd <= st)
      days = 1;			/* Minimum duration of one day */
    else
      days = (int)ceil(((double)nd / (60.0 * 60.0 * 24.0)) -
		       ((double)st / (60.0 * 60.0 * 24.0)));
  }
    
  return days;
}

/* Stand alone test harness */
int main(int argc, char *argv[])
{
  int ret = 0;

  if (argc == 4)
  {
    ret = duration(argv[1], argv[2], atoi(argv[3]));
    if (ret > 0)
      printf("Duration = %d days\n", ret);
    else
    {
      if (ret == -1)
        fprintf(stderr, "Incorrect format : should be YYYYMMDDHH24MI[SS]\n");
      else
        fprintf(stderr, "First date should precede second date\n");
      ret = 1;
    }
  }
  else if (argc == 3)
    printf("Julian = %ld\n", julian(argv[1], argv[2]));
  else if (argc == 2)
    printf("Julian = %ld\n", julian(argv[1], (char *)NULL));
  else
  {
    fprintf(stderr, "\nTwo formats are available :\n\n");
    fprintf(stderr, "%s <Start Date> <End Date> <allowance mins>\n", argv[0]);
    fprintf(stderr, "    Note : date formats are YYYYMMDDHH24MI[SS]\n");
    fprintf(stderr, "  OR\n");
    fprintf(stderr, "%s <Date> [<Format>]\n", argv[0]);

    return 1;
  }

  return 0;
}
